"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parseRefs = void 0;
var _immutable = require("immutable");
var _ReferencingHandler = require("@ui-schema/ui-schema/Plugins/ReferencingHandler");
var _SchemaRootProvider = require("@ui-schema/ui-schema/SchemaRootProvider");
var _getSchema = require("@ui-schema/ui-schema/Utils/getSchema");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(e, r, t) { return (r = _toPropertyKey(r)) in e ? Object.defineProperty(e, r, { value: t, enumerable: !0, configurable: !0, writable: !0 }) : e[r] = t, e; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : i + ""; }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var handleResolve = function handleResolve(keywords, condition, schema, context, recursive, pending) {
  Object.keys(keywords).forEach(function (keyword) {
    var schemaMap = schema.get(keyword);
    if (schemaMap && condition(schemaMap)) {
      schema = schema.set(keyword, schemaMap.map(function (subSchema) {
        var res = parseRefs(subSchema, context, keywords[keyword] || recursive, pending);
        pending = res.pending;
        return res.schema;
      }));
    }
  });
  return {
    schema: schema,
    pending: pending
  };
};
var checkNestedMapSchema = {
  properties: false
};
var checkNestedArraySchema = {};
var parseRefsInRenderingKeywords = function parseRefsInRenderingKeywords(schema, context, recursive) {
  var pending = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : (0, _immutable.Map)();
  var res = {
    schema: schema,
    pending: pending
  };
  res = handleResolve(checkNestedMapSchema, function (schemaMap) {
    return _immutable.Map.isMap(schemaMap);
  }, res.schema, context, recursive, res.pending);
  res = handleResolve(checkNestedArraySchema, function (schemaList) {
    return _immutable.List.isList(schemaList);
  }, res.schema, context, recursive, res.pending);
  return res;
};
var checkConditionalNestedMapSchema = {
  patternProperties: true,
  dependencies: false,
  dependentSchemas: false
};
var checkConditionalNestedArraySchema = {
  allOf: false,
  oneOf: false,
  anyOf: false,
  items: false
};
var checkSchema = {
  "if": true,
  "else": false,
  then: false,
  not: false,
  propertyNames: true,
  contains: true
};
var parseRefsInConditionalKeywords = function parseRefsInConditionalKeywords(schema, context) {
  var recursive = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var pending = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : (0, _immutable.Map)();
  var res = {
    schema: schema,
    pending: pending
  };
  Object.keys(checkSchema).forEach(function (keyword) {
    var schemaCond = res.schema.get(keyword);
    if (schemaCond && _immutable.Map.isMap(schemaCond)) {
      var resCheckSchema = parseRefs(schemaCond, context, checkSchema[keyword] || recursive, res.pending);
      res.schema = res.schema.set(keyword, resCheckSchema.schema);
      res.pending = resCheckSchema.pending;
    }
  });
  res = handleResolve(checkConditionalNestedMapSchema, function (schemaMap) {
    return _immutable.Map.isMap(schemaMap);
  }, res.schema, context, true, res.pending);
  res = handleResolve(checkConditionalNestedArraySchema, function (schemaList) {
    return _immutable.List.isList(schemaList);
  }, res.schema, context, recursive, res.pending);
  var items = res.schema.get('items');
  if (items && _immutable.Map.isMap(items)) {
    var itemsSchema = parseRefs(items, context, recursive, res.pending);
    res.schema = res.schema.set('items', itemsSchema.schema);
    res.pending = itemsSchema.pending;
  }
  return res;
};
var parseRefs = exports.parseRefs = function parseRefs(schema, context) {
  var recursive = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;
  var pending = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : (0, _immutable.Map)();
  var ref = schema.get('$ref');
  var schemaVersion = schema.get('version');
  if (ref) {
    try {
      var resolved = (0, _ReferencingHandler.resolveRef)(ref, context, schemaVersion);
      schema = resolved ? resolved.mergeDeep(schema["delete"]('version')["delete"]('$ref')) : schema;
    } catch (e) {
      if (e instanceof _ReferencingHandler.SchemaRefPending) {
        var id = context.id || '#';
        pending = pending.updateIn([id, ref], function () {
          var refPref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : (0, _immutable.List)();
          var v = schema.get('version') || '*';
          if (!refPref.contains(v)) {
            refPref = refPref.push(v);
          }
          return refPref;
        });
      } else {
        throw e;
      }
    }
  }
  if ((0, _SchemaRootProvider.isRootSchema)(schema)) {
    context = _objectSpread({}, context);
    context.id = (0, _getSchema.getSchemaId)(schema);
    context.root = schema;
    context.defs = schema.get('definitions') || schema.get('$defs');
  }
  var res = {
    schema: schema,
    pending: pending
  };
  res = parseRefsInConditionalKeywords(res.schema, context, recursive, res.pending);
  if (recursive) {
    res = parseRefsInRenderingKeywords(res.schema, context, recursive, res.pending);
  }
  return res;
};