"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.schemaLocalCachePath = exports.isLoaded = exports.UIApiProvider = exports.UIApiContext = exports.PROGRESS_START = exports.PROGRESS_NONE = exports.PROGRESS_ERROR = exports.PROGRESS_DONE = void 0;
exports.useProgress = useProgress;
exports.useUIApi = void 0;
var _react = _interopRequireDefault(require("react"));
var _immutable = require("immutable");
var _createMap = require("@ui-schema/ui-schema/Utils/createMap");
function _interopRequireDefault(e) { return e && e.__esModule ? e : { "default": e }; }
function _slicedToArray(r, e) { return _arrayWithHoles(r) || _iterableToArrayLimit(r, e) || _unsupportedIterableToArray(r, e) || _nonIterableRest(); }
function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function _iterableToArrayLimit(r, l) { var t = null == r ? null : "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (null != t) { var e, n, i, u, a = [], f = !0, o = !1; try { if (i = (t = t.call(r)).next, 0 === l) { if (Object(t) !== t) return; f = !1; } else for (; !(f = (e = i.call(t)).done) && (a.push(e.value), a.length !== l); f = !0); } catch (r) { o = !0, n = r; } finally { try { if (!f && null != t["return"] && (u = t["return"](), Object(u) !== u)) return; } finally { if (o) throw n; } } return a; } }
function _arrayWithHoles(r) { if (Array.isArray(r)) return r; }
var PROGRESS_NONE = exports.PROGRESS_NONE = false;
var PROGRESS_START = exports.PROGRESS_START = 'start';
var PROGRESS_DONE = exports.PROGRESS_DONE = true;
var PROGRESS_ERROR = exports.PROGRESS_ERROR = 'error';
function useProgress() {
  return _react["default"].useState(PROGRESS_NONE);
}
var UIApiContext = exports.UIApiContext = _react["default"].createContext({});
var useUIApi = exports.useUIApi = function useUIApi() {
  return _react["default"].useContext(UIApiContext);
};
var schemaLocalCachePath = exports.schemaLocalCachePath = 'uischema-cache' + (typeof window === 'undefined' ? 0 : window.location.port);
var initialState = function initialState(_ref) {
  var _ref$noCache = _ref.noCache,
    noCache = _ref$noCache === void 0 ? false : _ref$noCache;
  var cached;
  if (!noCache) {
    var _window;
    var cachedRaw = (_window = window) === null || _window === void 0 || (_window = _window.localStorage) === null || _window === void 0 ? void 0 : _window.getItem(schemaLocalCachePath);
    if (cachedRaw) {
      try {
        cached = JSON.parse(cachedRaw);
      } catch (e) {
        if (process.env.NODE_ENV === 'development') {
          console.error('invalid localStorage schema cache', e);
        }
      }
    }
  }
  return (0, _immutable.Map)({
    schemas: cached ? (0, _createMap.createOrderedMap)(cached) : (0, _immutable.Map)()
  });
};
function reducer(state) {
  var action = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  if (action.type === 'SCHEMA_LOADED') {
    return function () {
      var tmpState = state.setIn(['schemas', action.id], (0, _createMap.createOrderedMap)(action.value));
      if (!action.noCache) {
        var _window2;
        (_window2 = window) === null || _window2 === void 0 || (_window2 = _window2.localStorage) === null || _window2 === void 0 || _window2.setItem(schemaLocalCachePath, JSON.stringify(tmpState.get('schemas').toJS()));
      }
      return tmpState;
    }();
  }
  return state;
}
var schemasLoaded = {
  schemas: {}
};
var isLoaded = exports.isLoaded = function isLoaded(schemas, ref, version) {
  return !ref ? false : ref && (schemas === null || schemas === void 0 ? void 0 : schemas.get(ref)) && (version && (schemas === null || schemas === void 0 ? void 0 : schemas.getIn([ref, 'version'])) === version || (schemas === null || schemas === void 0 ? void 0 : schemas.getIn([ref, 'version'])) === '*' || !(schemas !== null && schemas !== void 0 && schemas.getIn([ref, 'version'])));
};
var UIApiProvider = exports.UIApiProvider = function UIApiProvider(_ref2) {
  var loadSchema = _ref2.loadSchema,
    noCache = _ref2.noCache,
    children = _ref2.children;
  var _React$useReducer = _react["default"].useReducer(reducer, {
      noCache: noCache
    }, initialState),
    _React$useReducer2 = _slicedToArray(_React$useReducer, 2),
    state = _React$useReducer2[0],
    dispatch = _React$useReducer2[1];
  var loadSchemaFn = _react["default"].useCallback(function (url) {
    if (schemasLoaded.schemas[url] === PROGRESS_START || schemasLoaded.schemas[url] === PROGRESS_DONE) return Promise.resolve(schemasLoaded.schemas[url]);
    schemasLoaded.schemas[url] = PROGRESS_START;
    return loadSchema(url).then(function (loadedSchema) {
      schemasLoaded.schemas[url] = PROGRESS_DONE;
      dispatch({
        type: 'SCHEMA_LOADED',
        id: url,
        value: loadedSchema,
        noCache: noCache
      });
      return PROGRESS_DONE;
    })["catch"](function () {
      schemasLoaded.schemas[url] = PROGRESS_ERROR;
      return PROGRESS_ERROR;
    });
  }, [loadSchema, noCache]);
  var contextValue = _react["default"].useMemo(function () {
    return {
      schemas: state.get('schemas'),
      loadSchema: loadSchemaFn
    };
  }, [state, loadSchemaFn]);
  return _react["default"].createElement(UIApiContext.Provider, {
    value: contextValue
  }, children);
};