"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.shouldDeleteOnEmpty = exports.prependKey = exports.createStore = exports.createEmptyStore = exports.addNestKey = exports.UIStore = void 0;
var _immutable = require("immutable");
var _schemaTypeIs = require("@ui-schema/ui-schema/Utils/schemaTypeIs");
var _UIStore = require("@ui-schema/ui-schema/UIStore");
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function _toConsumableArray(r) { return _arrayWithoutHoles(r) || _iterableToArray(r) || _unsupportedIterableToArray(r) || _nonIterableSpread(); }
function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _iterableToArray(r) { if ("undefined" != typeof Symbol && null != r[Symbol.iterator] || null != r["@@iterator"]) return Array.from(r); }
function _arrayWithoutHoles(r) { if (Array.isArray(r)) return _arrayLikeToArray(r); }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
var STR_INTERNALS = 'internals';
var STR_VALUES = 'values';
var STR_VALIDITY = 'validity';
var UIStore = exports.UIStore = (0, _immutable.Record)({
  values: undefined,
  internals: (0, _immutable.Map)(),
  validity: (0, _immutable.Map)(),
  meta: (0, _immutable.Map)(),
  valuesToJS: function valuesToJS() {
    var values = this.get(STR_VALUES);
    if (_immutable.Map.isMap(values) || _immutable.List.isList(values) || _immutable.Record.isRecord(values)) return values.toJS();
    return values;
  },
  getValues: function getValues() {
    return this.get(STR_VALUES);
  },
  getInternals: function getInternals() {
    return this.get(STR_INTERNALS);
  },
  getValidity: function getValidity() {
    return this.get(STR_VALIDITY);
  },
  extractValues: function extractValues(storeKeys) {
    return (0, _UIStore.doExtractValues)(storeKeys, this);
  }
});
var createStore = exports.createStore = function createStore(values) {
  return new UIStore({
    values: values,
    internals: (0, _immutable.Map)({
      internals: _immutable.List.isList(values) ? (0, _immutable.List)() : (0, _immutable.Map)()
    }),
    validity: (0, _immutable.Map)(),
    meta: (0, _immutable.Map)()
  });
};
var createEmptyStore = exports.createEmptyStore = function createEmptyStore() {
  var type = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'object';
  return createStore(type === 'array' ? (0, _immutable.List)([]) : type === 'string' ? '' : type === 'number' || type === 'integer' ? 0 : type === 'boolean' ? false : (0, _immutable.Map)());
};
var prependKey = exports.prependKey = function prependKey(storeKeys, key) {
  return Array.isArray(storeKeys) ? [key].concat(_toConsumableArray(storeKeys)) : storeKeys.splice(0, 0, key);
};
var shouldDeleteOnEmpty = exports.shouldDeleteOnEmpty = function shouldDeleteOnEmpty(value, force, type) {
  var valueTypeOf = _typeof(value);
  if (!force && !(0, _schemaTypeIs.schemaTypeIsNumeric)(type)) return false;
  if (valueTypeOf === 'undefined') return true;
  if ((0, _schemaTypeIs.schemaTypeIs)(type, 'string') && valueTypeOf === 'string' || (0, _schemaTypeIs.schemaTypeIs)(type, 'number') && (valueTypeOf === 'number' || valueTypeOf === 'string') || (0, _schemaTypeIs.schemaTypeIs)(type, 'integer') && (valueTypeOf === 'number' || valueTypeOf === 'string')) {
    return value === '' || valueTypeOf === 'undefined' || valueTypeOf === 'string' && 0 === value.trim().length;
  } else if ((0, _schemaTypeIs.schemaTypeIs)(type, 'boolean') && valueTypeOf === 'boolean') {
    return !value;
  } else if ((0, _schemaTypeIs.schemaTypeIs)(type, 'array') && (_immutable.List.isList(value) || Array.isArray(value))) {
    return _immutable.List.isList(value) && value.size === 0 || Array.isArray(value) && value.length === 0;
  } else if ((0, _schemaTypeIs.schemaTypeIs)(type, 'object') && (_immutable.Map.isMap(value) || _immutable.Record.isRecord(value) || valueTypeOf === 'object')) {
    return (_immutable.Map.isMap(value) || _immutable.Record.isRecord(value)) && value.keySeq().size === 0 || valueTypeOf === 'object' && Object.keys(value).length === 0;
  }
  return false;
};
var addNestKey = exports.addNestKey = function addNestKey(storeKeysNestedKey, storeKeys) {
  return storeKeysNestedKey ? storeKeys.reduce(function (nk, sk) {
    return nk === null || nk === void 0 ? void 0 : nk.concat(sk, (0, _immutable.List)([storeKeysNestedKey]));
  }, (0, _immutable.List)([storeKeysNestedKey])).splice(-1, 1) : storeKeys;
};