"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validateUniqueItems = exports.validateItems = exports.validateContains = exports.validateArrayContent = exports.validateAdditionalItems = exports.arrayValidator = exports.ERROR_NOT_FOUND_CONTAINS = exports.ERROR_MIN_CONTAINS = exports.ERROR_MAX_CONTAINS = exports.ERROR_DUPLICATE_ITEMS = exports.ERROR_ADDITIONAL_ITEMS = void 0;
var _immutable = require("immutable");
var _validateSchema = require("@ui-schema/ui-schema/validateSchema");
var _ValidatorErrors = require("@ui-schema/ui-schema/ValidatorErrors");
var _TypeValidator = require("@ui-schema/ui-schema/Validators/TypeValidator");
function _createForOfIteratorHelper(r, e) { var t = "undefined" != typeof Symbol && r[Symbol.iterator] || r["@@iterator"]; if (!t) { if (Array.isArray(r) || (t = _unsupportedIterableToArray(r)) || e && r && "number" == typeof r.length) { t && (r = t); var _n = 0, F = function F() {}; return { s: F, n: function n() { return _n >= r.length ? { done: !0 } : { done: !1, value: r[_n++] }; }, e: function e(r) { throw r; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var o, a = !0, u = !1; return { s: function s() { t = t.call(r); }, n: function n() { var r = t.next(); return a = r.done, r; }, e: function e(r) { u = !0, o = r; }, f: function f() { try { a || null == t["return"] || t["return"](); } finally { if (u) throw o; } } }; }
function _unsupportedIterableToArray(r, a) { if (r) { if ("string" == typeof r) return _arrayLikeToArray(r, a); var t = {}.toString.call(r).slice(8, -1); return "Object" === t && r.constructor && (t = r.constructor.name), "Map" === t || "Set" === t ? Array.from(r) : "Arguments" === t || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(t) ? _arrayLikeToArray(r, a) : void 0; } }
function _arrayLikeToArray(r, a) { (null == a || a > r.length) && (a = r.length); for (var e = 0, n = Array(a); e < a; e++) n[e] = r[e]; return n; }
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
var ERROR_DUPLICATE_ITEMS = exports.ERROR_DUPLICATE_ITEMS = 'duplicate-items';
var ERROR_NOT_FOUND_CONTAINS = exports.ERROR_NOT_FOUND_CONTAINS = 'not-found-contains';
var ERROR_MIN_CONTAINS = exports.ERROR_MIN_CONTAINS = 'min-contains';
var ERROR_MAX_CONTAINS = exports.ERROR_MAX_CONTAINS = 'max-contains';
var ERROR_ADDITIONAL_ITEMS = exports.ERROR_ADDITIONAL_ITEMS = 'additional-items';
var findDuplicates = function findDuplicates(arr) {
  return arr.filter(function (item, index) {
    return arr.indexOf(item) !== index;
  });
};
var validateArrayContent = exports.validateArrayContent = function validateArrayContent(itemsSchema, value) {
  var additionalItems = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
  var err = (0, _ValidatorErrors.createValidatorErrors)();
  var found = 0;
  if (_immutable.List.isList(itemsSchema) || Array.isArray(itemsSchema)) {
    if (_immutable.List.isList(value) || Array.isArray(value)) {
      if (!validateAdditionalItems(additionalItems, value, itemsSchema)) {
        err = err.addError(ERROR_ADDITIONAL_ITEMS, (0, _immutable.Map)({}));
        found++;
      }
    } else {
      err = err.addError(_TypeValidator.ERROR_WRONG_TYPE, (0, _immutable.Map)({
        actual: _typeof(value),
        arrayTupleValidation: true
      }));
      found++;
    }
  } else if (Array.isArray(value) || _immutable.List.isList(value)) {
    var _iterator = _createForOfIteratorHelper(value),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var val = _step.value;
        var tmpErr = (0, _ValidatorErrors.createValidatorErrors)();
        var tmpErr2 = (0, _validateSchema.validateSchema)(itemsSchema, val);
        if (tmpErr2.hasError()) {
          tmpErr = tmpErr.addErrors(tmpErr2);
        }
        if (tmpErr.errCount === 0) {
          found++;
        } else {
          err = err.addErrors(tmpErr);
        }
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }
  return {
    err: err,
    found: found
  };
};
var validateAdditionalItems = exports.validateAdditionalItems = function validateAdditionalItems(additionalItems, value, schema) {
  return additionalItems === true || additionalItems === false && (_immutable.List.isList(value) && value.size <= schema.size || Array.isArray(value) && value.length <= schema.size);
};
var validateItems = exports.validateItems = function validateItems(schema, value) {
  var items = schema.get('items');
  if (items && value) {
    var item_err = validateArrayContent(items, value, schema.get('additionalItems'));
    return item_err.err;
  }
  return (0, _ValidatorErrors.createValidatorErrors)();
};
var validateContains = exports.validateContains = function validateContains(schema, value) {
  var errors = (0, _ValidatorErrors.createValidatorErrors)();
  if (!(Array.isArray(value) || _immutable.List.isList(value))) return errors;
  var contains = schema.get('contains');
  if (!contains) return errors;
  var contains_type = contains.get('type');
  if (!contains_type) return errors;
  var minContains = schema.get('minContains');
  var maxContains = schema.get('maxContains');
  var item_err = validateArrayContent(contains, value, undefined);
  if (item_err.found < 1 && typeof minContains === 'undefined' && typeof maxContains === 'undefined' || typeof minContains === 'number' && minContains > item_err.found || typeof maxContains === 'number' && maxContains < item_err.found) {
    if (item_err.err.errCount !== 0) {
      errors = errors.addErrors(item_err.err);
    }
  }
  if (typeof minContains === 'number' && minContains > item_err.found) {
    errors = errors.addError(ERROR_MIN_CONTAINS, (0, _immutable.Map)({
      minContains: minContains
    }));
  }
  if (typeof maxContains === 'number' && maxContains < item_err.found) {
    errors = errors.addError(ERROR_MAX_CONTAINS, (0, _immutable.Map)({
      maxContains: maxContains
    }));
  }
  if (minContains !== 0 && (Array.isArray(value) && value.length === 0 || _immutable.List.isList(value) && value.size === 0)) {
    errors = errors.addError(ERROR_NOT_FOUND_CONTAINS);
  }
  return errors;
};
var validateUniqueItems = exports.validateUniqueItems = function validateUniqueItems(schema, value) {
  var uniqueItems = schema.get('uniqueItems');
  if (uniqueItems && (_immutable.List.isList(value) || Array.isArray(value))) {
    var duplicates = findDuplicates(value);
    if (Array.isArray(duplicates)) {
      return duplicates.length === 0;
    } else if (_immutable.List.isList(duplicates)) {
      return duplicates.size === 0;
    }
  }
  return true;
};
var arrayValidator = exports.arrayValidator = {
  should: function should(_ref) {
    var value = _ref.value;
    return _immutable.List.isList(value) || Array.isArray(value);
  },
  handle: function handle(_ref2) {
    var schema = _ref2.schema,
      value = _ref2.value,
      errors = _ref2.errors,
      valid = _ref2.valid;
    var uniqueItems = schema.get('uniqueItems');
    if (uniqueItems && value) {
      if (!validateUniqueItems(schema, value)) {
        valid = false;
        errors = errors.addError(ERROR_DUPLICATE_ITEMS);
      }
    }
    var items = schema.get('items');
    if (items && value) {
      var items_err = validateItems(schema, value);
      if (items_err.hasError()) {
        valid = false;
        errors = errors.addErrorsToChild(items_err);
      }
    }
    var contains = schema.get('contains');
    if (contains && value) {
      var containsError = validateContains(schema, value);
      if (containsError.hasError()) {
        valid = false;
        errors = errors.addErrors(containsError);
      }
    }
    return {
      errors: errors,
      valid: valid
    };
  }
};